% Kristoffer Aasland
% My first try making a path planner using a potential field for a
% two link revolute joint manipulator

clear all;
zeta = [5 3];           % Uatt potential field factor
h=1;                    % Potential field Resolution
obsLimit = 3;           % Limit for Urep potential field
goalLimit = 2;          % Limit for conic potential field
eta = 10;               % Urep potential field factor

epsi = [0.1 0.1]';      %limit to goal in descent algorithm
alpha =0.09;            %step length in descent algorithm

loop = 120;

a1 = 3;
a2 = 2.5;

startTheta = [ pi/12  0]';
goalTheta  = [ (3*pi/4) 0]';

[startX, startY] = forKin(a1,a2,startTheta(1),startTheta(2));
[goalX, goalY] = forKin(a1,a2,goalTheta(1),goalTheta(2));


squareX = [5   5];
squareY = [5.5 6];

% Gradient Descent Algorithm

int = 2;
pointX = startX ;
pointY = startY ;
theta = startTheta;

pathX(:,1) = pointX;
pathY(:,1) = pointY;
len=length(startX);

manipx = [5 pointX' ];
manipy = [0 pointY' ];
    
figure(1)
axis([0 11 ,-2 11]) ;
plot([5 goalX'], [0 goalY'])
hold on
plot(manipx, manipy);
hold off
hei = 1;


while abs( goalTheta(1)-theta(1) ) > epsi(1) || abs( goalTheta(2)-theta(2) ) > epsi(2)
    
    %Calculating the Jacobians:    
    
       JacobiTrans(:,:,1) = jaTrans01(a1,a2,theta(1),0); 
       JacobiTrans(:,:,2) = jaTrans02(a1,a2,theta(1),theta(2)) ;
   
    % Calcuatling the attractive field Uatt:
    for k=1:len
        
        Dist = sqrt( (goalX(k)-pointX(k))^2 + (goalY(k)-pointY(k))^2 );
        
        if Dist < goalLimit
            pxa(k) = -zeta(k)*(pointX(k)-goalX(k)); 
            pya(k) = -zeta(k)*(pointY(k)-goalY(k));
        else
           
            pxa(k)    = -goalLimit*zeta(k)*( (pointX(k)-goalX(k))/Dist );
            pya(k)    = -goalLimit*zeta(k)*( (pointY(k)-goalY(k))/Dist );
        end
            
        TorAtt(:,k) = JacobiTrans(:,:,k)*[pxa(k) pya(k)]';
    end
    % Calculating the repulsive field Urep: 
    
    Urep = 0;
    Ur = 0;
    pyrTemp = 0;
    pxrTemp = 0;
    pyr = 0;
    pxr = 0;
    TorRep = TorAtt*0;

    for k=1:len
        
        for i=1:(length(squareX))
    
            obst = [squareX(i);   squareY(i) ];   
            Dist = sqrt( (obst(1)-pointX(k)).^2 + (obst(2)-pointY(k)).^2 );

              if Dist < obsLimit 

                    if Dist< 0.001 && Dist>=0
                        Dist = 0.001; 
                    elseif Dist> -00.01 && Dist<=0
                        Dist = -0.001;                
                    end

                    Ur = 0.5*eta*(( 1/Dist - 1/obsLimit )^2);
                    pxrTemp = -eta*( 1/obsLimit - 1/Dist ) * ( 1/(Dist^2) ) * (( pointX(k)-obst(1) )/Dist); % * (( X(i,j)-obst(1) )/Dist(i,j));
                    pyrTemp = -eta*( 1/obsLimit - 1/Dist ) * ( 1/(Dist^2) ) * (( pointY(k)-obst(2) )/Dist); % * (( Y(i,j)-obst(2) )/Dist(i,j)); 
                    TorRep(:,k) = TorRep(:,k) + JacobiTrans(:,:,k)* [pxrTemp pyrTemp]';
                    if Ur > 9999
                        Ur = 50;
                    end

                else
                    Ur= 0;
                   
              end

            Urep = Urep + Ur;
            Ur = 0;
           
            pxrTemp = 0;
            pyrTemp = 0;
        end
        Urepul(k) = Urep ;
        
    end  
                
    %Sumarizeing the attractive and repulsive Torques
    
    sumTor = TorAtt(:,1)*0;
    for k=1:len
        sumTor = sumTor + TorAtt(:,k) + TorRep(:,k);
    end
    
    theta = theta + alpha*sumTor/norm(sumTor);
   
    % PLOT
    [pointX, pointY] = forKin( a1, a2, theta(1), theta(2) );
    
    manipx = [5 pointX' ];
    manipy = [0 pointY' ];
    
    figure(1)
    axis([0 11 ,-2 11]) ;
    hold on
    plot(manipx, manipy);
 hei = hei +1;   
end

plot(squareX,squareY,'o')
hold off





