% 3D Path Planning and visualization
clear all;
figure(1)
hold off
plot3(0,0,0,'o')
hold on

%% Initalzing
zeta = [4 0.5 0.5 2];      % Uatt potential field factor
obsLimit = 10;             % Limit for Urep potential field
goalLimit = 2;             % Limit for conic potential field
eta =3;                    % Urep potential field factor

alpha =0.1;               %step length in descent algorithm

mainRuns = 200;          %A for loop is used in the path planner instead of a
                         %while loop that migth run to infinity

%% Making some obstacles    
Obstacles = [3 6 3  
             3 6 3 %];
             2 2 2
             10 1 10];

ObsMat= [0     0    0     6   0 20
         0     0    0     10   0   8%];
         0     0    0     -3.74   0   18%] 
         0     0    0      0   7   10];

numObs =size(Obstacles);

    %Making transformation Matrices for the obstacles
[oT oRot oTrans] = obsTransformation(ObsMat);

 % Making some obstacle objects for visualization
[xobs1 yobs1 zobs1] = obsTransf( Obstacles(1,1), Obstacles(1,2), Obstacles(1,3), oRot(:,:,1), oTrans(:,:,1) );      
[xobs2 yobs2 zobs2] = obsTransf( Obstacles(2,1), Obstacles(2,2), Obstacles(2,3), oRot(:,:,2), oTrans(:,:,2) );      
[xobs3 yobs3 zobs3] = obsTransf( Obstacles(3,1), Obstacles(3,2), Obstacles(3,3), oRot(:,:,3), oTrans(:,:,3) );      
[xobs4 yobs4 zobs4] = obsTransf( Obstacles(4,1), Obstacles(4,2), Obstacles(4,3), oRot(:,:,4), oTrans(:,:,4) );      

 % Drawing the obstacles
patch(xobs1,  yobs1,  zobs1,  [0.7 0.5 0.2]);
patch(xobs2,  yobs2,  zobs2,  [0.7 0.5 0.2]);
patch(xobs3,  yobs3,  zobs3,  [0.7 0.5 0.2]);
patch(xobs4,  yobs4,  zobs4,  [0.7 0.5 0.2]);
%% Making the the 4-link robot

% Specify the joint angle lmitis

JALimits = pi/180* [-180    180
                    -100-90 110-90
                     -65     60
                    -180    180];   

links = 4;
Manip = 0.5*[ 5.10 6.15 5.10 
              7.05 2.00 2.00
              2.50 2.00 3.60
              1.50 9.35 1.50  ];
          
Man = 0.5*[ 0.00 6.15 0.00
            7.05 0.00 0.00
            0.00 0.00 3.60
            0.00 9.35 0.00  ];          


    % Final pose
thf1=   0;
thf2=   pi/6;
thf3=   -pi/8; 
thf4=   0;

 % Final Denavit-Hartenberg matrix
 DenHaf = [thf1       6.15  1.00  -pi/2
           thf2-pi/2  0.00  7.05  0
           thf3       0.00  1.35  -pi/2
           thf4       7.55  0.00  pi/2]; 
        
  
  % Making transformation matrices for the final joint frames
[ffT, ffRot, ffTrans ] = dh(DenHaf);
  % Making transformation matrices for the final links
[lfT, lfRot, lfTrans ] = dhLinkABB(ffT,Manip,Man);
    
    %%%%%% Initial pose %%%%%%%
th1=   pi/2;
th2=  -pi/4;
th3=  -pi/3;
th4=  0;

   DH = [th1       6.15  1.00  -pi/2
      th2-pi/2  0.00  7.05  0
      th3       0.00  1.35  -pi/2
      th4       7.55  0.00  pi/2];
  
  theta = [DH(1,1) DH(2,1) DH(3,1) DH(4,1) ]';
  
  % Making transformation matrices for the joint frames
[fT, fRot, fTrans ] = dh(DH);
  % Making transformation matrices for the links
[lT, lRot, lTrans ] = dhLinkABB(fT,Manip,Man);

        % Making the link objects in initial pose for visualization

[xlink1i ylink1i zlink1i] = linkTransf( Manip(1,1), Manip(1,2), Manip(1,3) , lRot(:,:,1), lTrans(:,:,1) );
[xlink2i ylink2i zlink2i] = linkTransf( Manip(2,1), Manip(2,2), Manip(2,3) , lRot(:,:,2), lTrans(:,:,2) );
[xlink3i ylink3i zlink3i] = linkTransf( Manip(3,1), Manip(3,2), Manip(3,3) , lRot(:,:,3), lTrans(:,:,3) );   
[xlink4i ylink4i zlink4i] = linkTransf( Manip(4,1), Manip(4,2), Manip(4,3) , lRot(:,:,4), lTrans(:,:,4) );   


hallo = 0;
indi=1;
%%%%%%% Starting the for loop to compute the path %%%%%%
%% Starting the for loop

for i=1:mainRuns% 95 funker fint
    hallo = hallo + 1;
    % Making transformation matrices for the joint frames
    [fT, fRot, fTrans ] = dh(DH);
    % Making transformation matrices for the links
    [lT, lRot, lTrans ] = dhLinkABB(fT,Manip,Man);


%% Calculate the transposed Jacobian of "fixed" length

    fJacobianABBT(:,:,1) = jacobianABBT01(fT);   
    fJacobianABBT(:,:,2) = jacobianABBT02(fT);   
    fJacobianABBT(:,:,3) = jacobianABBT03(fT);
    fJacobianABBT(:,:,4) = jacobianABBT04(fT);    
    
%% Calculating the Attractive field (only for end effector?)
    attTorque(1:links,1) =zeros;
    for k=1:links
        
        dist = norm ( ffT(1:3,4,k)-fT(1:3,4,k) );
        
        if dist < goalLimit           
            pa(:,k)  = -zeta(k)*( fT(1:3,4,k) - ffT(1:3,4,k) );
        else           
            pa(:,k)   = -goalLimit*0.5*zeta(k)*( fT(1:3,4,k) - ffT(1:3,4,k) )/dist; 
        end
            
        attTorque = attTorque + fJacobianABBT(:,:,k)*pa(:,k);
    end
    
   
%% Calculating closest distances between links and obstacles 
 
    it = 0;
    repTorque(1:links,1) =zeros;
    for l=1:links        
        for o =1:numObs(1)
            it = it+1;
             %Starting the GJK algorithm
             %Initializing
            
              % Finding an initial vector inside A-B using the Supportpoints             
             avec = [ 2 6 3]';
             wR   = supportPoint(-avec,  Manip(l,:), lRot(:,:,l), lTrans(:,:,l));   
             wObs = supportPoint(avec, Obstacles(o,:), oRot(:,:,o), oTrans(:,:,o));  
             a = wR-wObs; 
             
             bvec = [ 2 -6 3]';
             wR   = supportPoint(-bvec,  Manip(l,:), lRot(:,:,l), lTrans(:,:,l));   
             wObs = supportPoint(bvec, Obstacles(o,:), oRot(:,:,o), oTrans(:,:,o));  
             b = wR-wObs;             
             
             c = a-b;
             vInit = b + 0.5*c;       

             %Extract the disatance, direction, closest point on R obstacle
             [inter dist v r obs ] = gjk(vInit, Manip(l,:), lRot(:,:,l), lTrans(:,:,l), Obstacles(o,:), oRot(:,:,o), oTrans(:,:,o) );
             
              if inter==1   
                 plot3([r(1) obs(1)], [r(2) obs(2)], [r(3) obs(3)],'r');
                 plot3(r(1),r(2),r(3),'h k');
                 plot3(obs(1),obs(2),obs(3),'h k');
                 disp(' **************INTERSECTION***************************************************************************************')
                 break;
             end
             
             if dist<0.001
                 dist
                 error(' possible intersection')
                
             end
             
%              
             
             %%%%%%%% Calculate the "variable" transposed Jacobian %%%%%  
            if l ==1                        
                 JacobianABBTVar(:,:,1) = jacobianABBTVar01NY(fT,r);                
             elseif l==2                
                 JacobianABBTVar(:,:,2) = jacobianABBTVar02NY(fT,r);
             elseif l==3    
                 JacobianABBTVar(:,:,3) = [0 0 0 
                                           0 0 0 
                                           0 0 0 
                                           0 0 0 ];
             elseif l==4%                           
                 JacobianABBTVar(:,:,4) = jacobianABBTVar04NY(fT,r);
               
             end
             
             %%%%%%% Calculate the Repulisve field %%%%%%
             if dist < obsLimit
                 Ur = 0.5*eta*(( 1/dist - 1/obsLimit )^2);                 
                 repForce = -eta*( 1/obsLimit - 1/dist ) * ( 1/(dist^2) ) * ((r-obs )/dist); % * (( Y(i,j)-obst(2) )/Dist(i,j));
                 repTorque = repTorque + JacobianABBTVar(:,:,l)* repForce';                         
             end             
        end 
             if inter==1                 
                 disp(' **************INTERSECTION***************')
                 break;
             end
             
    end
    if inter==1                 
        disp(' **************INTERSECTION***************')
        break;
    end
%% Summarize attractive and repulsive torques
     
    sumTorque = attTorque + repTorque;
    
     
%%  Updating the new theta values
    if norm(sumTorque) < 3;
            theta = theta + alpha*sumTorque/norm(sumTorque); % 3*alpha*sumTor;
        else
            theta = theta + alpha*sumTorque/norm(sumTorque);
    end
    
    if theta(1)     < JALimits(1,1)
        theta(1) =    JALimits(1,1);
    elseif theta(1) > JALimits(1,2)
        theta(1) =    JALimits(1,2);
    elseif theta(2) < JALimits(2,1)
        theta(2) =    JALimits(2,1);
    elseif theta(2) > JALimits(2,2)
        theta(2) =    JALimits(2,2);
    elseif theta(3) < JALimits(3,1)
        theta(3) =    JALimits(3,1);   
    elseif theta(3) > JALimits(3,2)
        theta(3) =    JALimits(3,2);
    elseif theta(4) < JALimits(4,1)
        theta(4) =    JALimits(4,1);
    elseif theta(4) > JALimits(4,2)
        theta(2) =    JALimits(4,2);
    end
    
    %     %Collecting waypoints
    if hallo == 25
        WP(:,indi) = theta;
       indi = indi+1;
        hallo = 0;
    end
    
    DH(1,1) = theta(1);
    DH(2,1) = theta(2);
    DH(3,1) = theta(3);
    
    plot3( [fT(1,4,1) fT(1,4,2) fT(1,4,3) fT(1,4,4)],[fT(2,4,1) fT(2,4,2) fT(2,4,3) fT(2,4,4) ],[fT(3,4,1) fT(3,4,2) fT(3,4,3) fT(3,4,4)] )
    
%         % Making the link objects in current pose for visualization
  
[xlink1 ylink1 zlink1] = linkTransf( Manip(1,1), Manip(1,2), Manip(1,3) , lRot(:,:,1), lTrans(:,:,1) );
[xlink2 ylink2 zlink2] = linkTransf( Manip(2,1), Manip(2,2), Manip(2,3) , lRot(:,:,2), lTrans(:,:,2) ); % -  fRot(:,:,2)*[Manip(2,1) 0 0 ]' ) );
[xlink3 ylink3 zlink3] = linkTransf( Manip(3,1), Manip(3,2), Manip(3,3) , lRot(:,:,3), lTrans(:,:,3) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    
[xlink4 ylink4 zlink4] = linkTransf( Manip(4,1), Manip(4,2), Manip(4,3) , lRot(:,:,4), lTrans(:,:,4) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    

% Drawing the manipulator in its current pose
patch(xlink1, ylink1, zlink1, [0 0 1]);
patch(xlink2, ylink2, zlink2, [0 0 1]);
patch(xlink3, ylink3, zlink3, [0 0 1]);
patch(xlink4, ylink4, zlink4, [0 0 1]);
    
end    
    
    
    
%% Drawing the output

    % Making the link objects in final pose for visualization

[xlink1f ylink1f zlink1f] = linkTransf( Manip(1,1), Manip(1,2), Manip(1,3) , lfRot(:,:,1), lfTrans(:,:,1) );
[xlink2f ylink2f zlink2f] = linkTransf( Manip(2,1), Manip(2,2), Manip(2,3) , lfRot(:,:,2), lfTrans(:,:,2) ); % -  fRot(:,:,2)*[Manip(2,1) 0 0 ]' ) );
[xlink3f ylink3f zlink3f] = linkTransf( Manip(3,1), Manip(3,2), Manip(3,3) , lfRot(:,:,3), lfTrans(:,:,3) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    
[xlink4f ylink4f zlink4f] = linkTransf( Manip(4,1), Manip(4,2), Manip(4,3) , lfRot(:,:,4), lfTrans(:,:,4) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    
    
     % Making the link objects for visualization
  
[xlink1 ylink1 zlink1] = linkTransf( Manip(1,1), Manip(1,2), Manip(1,3) , lRot(:,:,1), lTrans(:,:,1) );
[xlink2 ylink2 zlink2] = linkTransf( Manip(2,1), Manip(2,2), Manip(2,3) , lRot(:,:,2), lTrans(:,:,2) ); % -  fRot(:,:,2)*[Manip(2,1) 0 0 ]' ) );
[xlink3 ylink3 zlink3] = linkTransf( Manip(3,1), Manip(3,2), Manip(3,3) , lRot(:,:,3), lTrans(:,:,3) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    
[xlink4 ylink4 zlink4] = linkTransf( Manip(4,1), Manip(4,2), Manip(4,3) , lRot(:,:,4), lTrans(:,:,4) ); %- fRot(:,:,3)* [ Manip(3,1) 0 0]' )); % - [0 DH(3,3)/2 DH(3,2)/2]' ) );    


figure(1)

% Drawing the manipulator in its initial pose
patch(xlink1i, ylink1i, zlink1i, [0 1 0]);
patch(xlink2i, ylink2i, zlink2i, [0 1 0]);
patch(xlink3i, ylink3i, zlink3i, [0 1 0]);
patch(xlink4i, ylink4i, zlink4i, [0 1 0]);

 % Drawing the maipulttor in its final pose
patch(xlink1f, ylink1f, zlink1f, [1 0 0]);
patch(xlink2f, ylink2f, zlink2f, [1 0 0]);
patch(xlink3f, ylink3f, zlink3f, [1 0 0]);
patch(xlink4f, ylink4f, zlink4f, [1 0 0]);

 % Drawing the manipulator in its current pose
patch(xlink1, ylink1, zlink1, [0 0 1]);
patch(xlink2, ylink2, zlink2, [0 0 1]);
patch(xlink3, ylink3, zlink3, [0 0 1]);
patch(xlink4, ylink4, zlink4, [0 0 1]);

   

axis([-20 20 -20 20 0 40]);
% legend('green initial position')
hold off

