% Manipulator visualization
clear all;
figure(1)
hold off
plot3(0,0,0,'o');
hold on

% Making some obstacles    
Obstacles = [1 1 1
             2 2 2
             1 5 1
             1 10 10];
    
ObsMat= [pi/6 pi/3 pi/2   -4   -1   15
         pi/2  0    0     0   0   20
         pi/4 pi/4 pi/6 -4   6   15 
         0     0    0     5  0  10];
 
[oT oRot oTrans] = obsTransformation(ObsMat);

% Specifying the robot

Manip = 0.5*[ 5.10 6.15 5.10 
              7.05 2.00 2.00
              2.50 2.00 3.60
              1.50 9.35 1.50  ];
          
Man = 0.5*[ 0.00 6.15 0.00
            7.05 0.00 0.00
            0.00 0.00 3.60
            0.00 9.35 0.00  ];   
    % Defining the workspace limits
JALim = pi/180* [-180 180
                 -100 110
                  -65  60
                 -180 180];   
% Intitalize joint angles         

% th1Init = JALim(1,1);
% th2Init = JALim(2,1);
% th3Init = JALim(3,1);
% th4Init = JALim(4,1); 
stepLength = 45/180*pi;

 % Starting the Probabilistic RoadMap method              
%Config(1:8,1:4,4,1000) = zeros;
con = 1;
for th1=JALim(1,1):30/180*pi:JALim(1,2)
    for th2=JALim(2,1):stepLength:JALim(2,2)
        for th3=JALim(3,1):stepLength:JALim(3,2)
            for th4=JALim(4,1):stepLength:JALim(4,2)

                DH = [th1       6.15  1.00  -pi/2
                      th2-pi/2  0.00  7.05  0
                      th3       0.00  1.35  -pi/2
                      th4       7.55  0.00  pi/2];

                % Making transformation matrices for the joint frames
                [fT, fRot, fTrans ] = dh(DH);
                % Making transformation matrices for the links
                [lT, lRot, lTrans ] = dhLinkABB(fT,Manip,Man);

                % Checking for collision  between links and obstacles
                it = 0;
                for l=1:4

                    for o =1:4
                        it = it+1;
                        %Starting the GJK algorithm
                        %Initializing

                        vInit = [ 2 -6 3]';
                        %Check for collision
                        inter  = gjkInter(vInit, Manip(l,:), lRot(:,:,l), lTrans(:,:,l), Obstacles(o,:), oRot(:,:,o), oTrans(:,:,o) );
                        if inter
                            break;
                        end
                    end
                    
                    if inter
                        break;
                    end                    
                end
                % if no collision detected in current configuration, store
                % the configuration
                if inter == false
                    
                        Config(1:4, 1:4,:,con) = fT;
                        Config(5:8, 1:4,:,con) = lT;
                        con = con + 1 ;
                        % plot the configuration
                        plot3( [fT(1,4,1) fT(1,4,2) fT(1,4,3) fT(1,4,4)],[fT(2,4,1) fT(2,4,2) fT(2,4,3) fT(2,4,4) ],[fT(3,4,1) fT(3,4,2) fT(3,4,3) fT(3,4,4)] )
                end
            end %end theta4 loop 
        end %end theta3 loop 
    end %end theta2 loop 
end %end theta1 loop    
    
% Making the links for displaying
  
[xlink1 ylink1 zlink1] = linkTransf( Manip(1,1), Manip(1,2), Manip(1,3) , lRot(:,:,1), lTrans(:,:,1) );
[xlink2 ylink2 zlink2] = linkTransf( Manip(2,1), Manip(2,2), Manip(2,3) , lRot(:,:,2), lTrans(:,:,2) );
[xlink3 ylink3 zlink3] = linkTransf( Manip(3,1), Manip(3,2), Manip(3,3) , lRot(:,:,3), lTrans(:,:,3) );  
[xlink4 ylink4 zlink4] = linkTransf( Manip(4,1), Manip(4,2), Manip(4,3) , lRot(:,:,4), lTrans(:,:,4) );  

    
[xobs1 yobs1 zobs1] = obsTransf( Obstacles(1,1), Obstacles(1,2), Obstacles(1,3), oRot(:,:,1), oTrans(:,:,1) );      
[xobs2 yobs2 zobs2] = obsTransf( Obstacles(2,1), Obstacles(2,2), Obstacles(2,3), oRot(:,:,2), oTrans(:,:,2) );      
[xobs3 yobs3 zobs3] = obsTransf( Obstacles(3,1), Obstacles(3,2), Obstacles(3,3), oRot(:,:,3), oTrans(:,:,3) ); 
[xobs4 yobs4 zobs4] = obsTransf( Obstacles(4,1), Obstacles(4,2), Obstacles(4,3), oRot(:,:,4), oTrans(:,:,4) );              
    
% Drawing the output
figure(1)
patch(xlink1, ylink1, zlink1, [1 0 0]);
hold on
patch(xlink2, ylink2, zlink2, [1 0 0]);
patch(xlink3, ylink3, zlink3, [1 0 0]);
patch(xlink4, ylink4, zlink4, [1 0 0]);

patch(xobs1,  yobs1,  zobs1,  [0 1 0]);
patch(xobs2,  yobs2,  zobs2,  [0 1 0]);
patch(xobs3,  yobs3,  zobs3,  [0 1 0]);
patch(xobs4,  yobs4,  zobs4,  [0 1 0]);

axis([-20 20 -20 20 0 40]);
hold off

