% Make samples

function [Samples comp] = makeBridgeSamples_all_Nearest_k(K, LinLimits, JALimits,linStepLength, angularStepLength, w, step, Obstacles, oRot, oTrans)
% function [coverage, TotalCoverage, sumQueriesConnection, TestResultConnection, TotalConnectionTime, TotalPathSearchTime, sampleTime ] = makeBridgeSamples_Nearest_k(K, LinLimits, JALimits,linStepLength, angularStepLength,w,step, Obstacles, oRot, oTrans)
clear comp
clear samples
clear coverage
clear totalCoverage
clear sumQueriesConnection
clear TestResultConnection
clear TotalConnectionTime 
clear sampleTime
stopCon = 50;
stopInterval = 50;
maxSamples = 101;

alternateSampleInterval = 10;
useTempSampler = false;
stupidConfigs = [];

[obsN notInUse2] = size(Obstacles);

%%%%%%%% Starting the Probabilistic RoadMap method%%%%%%%              
%Config(1:8,1:4,4,1000) = zeros;
comp(1) = struct('Node',[1], 'NB',[0]);
emptyComp(1) = 2;

% stepVec = [6 5 4 3 2 1 1];
% adaptive=false;
% if step==8
%     adaptive=true;
%     step=1;
% end

con = 1;
samplesStored=0;
tic %Start sampling timer
for pos =1:1000000
    
    %%%%%%% Make a new random dummy robot configuration%%%%%%%%%%%%
    [ManipD, ManD, fTD, lTD, lRotD, lTransD, DHD, linValuesD, thetaD] =...
        makeRandomRobotConfig();
    [linksN notInUse1] = size(ManipD);

    % Checking for collision  between links and obstacles
    collisionDummy = collisionCheckABB(linksN, obsN, ManipD, lRotD, lTransD, Obstacles, oRot, oTrans);
    
    if ~mod(con,alternateSampleInterval)
        useTempSampler = true;
    end
    % If the robot manipulator collide, make a new neighbour sample
    if collisionDummy
        % Store Dummy 1 configuration
        configD(2:4,1) = linValuesD;
        configD(5:10,1)= thetaD;
        
        % Make Dummy2 configuration
        configD2 = moveRandStep(step,w,configD, LinLimits, JALimits);
        
        linValuesD2 = configD2(2:4);
        thetaD2 = configD2(5:10);

        [ManipD2, ManD2, fTD2, lTD2, lRotD2, lTransD2, DHD2, LinLimits, JALimits] = makeRobot(linValuesD2, thetaD2 );
        collisionDummy2 = collisionCheckABB(linksN, obsN, ManipD2, lRotD2, lTransD2, Obstacles, oRot, oTrans);
        
        if collisionDummy2
%             disp('Found a close neighbour configuration')
            % Make mid config between config dummy1 and config dummy2
            midConfig = configD2 + ( (configD-configD2)/2 ) ;
            
            linValues = midConfig(2:4);
            theta = midConfig(5:10);
            
            [Manip, Man, fT, lT, lRot, lTrans, DH, LinLimits, JALimits] = makeRobot(linValues, theta );
            collision = collisionCheckABB(linksN, obsN, Manip, lRot, lTrans, Obstacles, oRot, oTrans);
            
            % if no collision detected in current configuration, store
            % the configuration
            if collision == false
                samplesStored = samplesStored +1;
%                 disp('Found a configuration between the bridge ')
%                             drawRobot(1,[0 1 0], linksN, ManipD, lRotD, lTransD)
%                             drawRobot(1,[0 0 1], linksN, ManipD2, lRotD2, lTransD2)
%                             drawRobot(1,[1 0 0], linksN, Manip, lRot, lTrans)
                %             drawGantryCrane(1, fT, linValues(1))
                %                     theta = [Th1 Th2 Th3 Th4]';
                Samples(1,con) = 0;
                Samples(2:4,con) = linValues;
                Samples(5:10,con)= theta;
                Samples(11,con) = 0;

                %         % plot the configuration
                %         plot3( [fT(1,4,1) fT(1,4,2) fT(1,4,3) fT(1,4,4)],[fT(2,4,1) fT(2,4,2) fT(2,4,3) fT(2,4,4) ],[fT(3,4,1) fT(3,4,2) fT(3,4,3) fT(3,4,4)] )               
               
                if con == 1
                    Samples(11,con)=1; %Sample 1 belongs to component 1
                else
                    clear TempMat;
                    for teller = 1:con-1 %Computing distance to all nodes in  one component
                        TempMat(teller,:) = [configDist([ 0; linValues; theta], Samples(1:10,teller), w ),teller ];
                    end

                    % Sort the matrix with increasing distance
                    clear sortMat;
                    sortMat = sortrows(TempMat);
                    % Try make a connection between current node
                    % and closest node
                    [rows col] = size(sortMat);
                    if rows>K
                        rows = K;
                    end

                    Config(1:10,1) = [0; linValues; theta ];
                    for ind=1:rows
                        c = Samples(11,sortMat(ind,2));
                        %                                     Config(1:10,1) = [0; linValues; theta ];
                        Config(1:10,2) = Samples(1:10, sortMat(ind,2));
                        interaction = binCollisionCheckABB( Config,linStepLength, angularStepLength,linksN, obsN, Obstacles, oRot, oTrans);
                        if ~interaction % If no interaction, add the node to the component
                            [ connectSuccess Samples comp emptyComp] = connectNodeABB_K(con, c, ind, Samples, comp, sortMat, emptyComp);
%                             if connectSuccess % Node added to component break for loop
%                                 break;
%                             end
                        end
                    end % If no connection found , try next sample.

                    % if component connection failed, make a new component
                    [Samples comp emptyComp] = connectToEmptyComponentABB(con,Samples, comp, emptyComp);
                end
                con = con + 1
                
                if con == maxSamples
                    disp('found 100 samples')
                    return
                end

%                 if con==stopCon % Do some testing
%                     con
%                     interval = con/stopInterval;
%                     %%%%%%%% TESTING, STOP SAMPLE TIMER%%%%%%
%                     sampleTime(interval) = toc;                   
%                                         
%                     %% Test the probabilistic road map
%                     [testConfigCovered, TestQueryConnection, ConnectionTime, PathSearchTime, nTestNodes ] = PRM_SamplingAnalysis(comp, Samples, w);
% 
%                     % Storing the data
% 
%                     if interval==1
%                         TotalCoverage = testConfigCovered;
%                         TestResultConnection = TestQueryConnection;
%                         TotalConnectionTime(:,:,interval) = ConnectionTime;
%                         TotalPathSearchTime(:,:,interval) = PathSearchTime;
%                     else
%                         TotalCoverage = TotalCoverage + testConfigCovered;
%                         TestResultConnection = TestResultConnection + TestQueryConnection;
%                         TotalConnectionTime(:,:,interval) = ConnectionTime;
%                         TotalPathSearchTime(:,:,interval) = PathSearchTime;
%                     end
% 
%                     coverage(interval) = sum(testConfigCovered);
% 
%                     sumColums = sum(TestQueryConnection);
%                     sumQueriesConnection(interval) = sum(sumColums);
% 
%                     stopCon = stopCon + stopInterval;
%                     if stopCon == maxSamples
%                         disp('Reached 300 samples')
%                         return
%                     end
%                     if adaptive
%                        step = stepVec(interval);
%                     end
%                     tic %Start new smaple timer
%                 end %Done testing and collecting data

            end
        end %end collisionDummy2
    end %end the collisionDummy
end %end the pos loop
samplesStored;
disp('Done with Bridge sampling')
