% Make samples

% function [Samples comp] = makeGaussianSamples_Nearest_k(K, LinLimits, JALimits, stepLength,w,step, Obstacles, oRot, oTrans)
function [coverage, TotalCoverage, sumQueriesConnection, TestResultConnection, TotalConnectionTime, TotalPathSearchTime, sampleTime, TotalComponents ] = makeGaussianSamplesTest_Nearest_k(K, LinLimits, JALimits, linStepLength, angularStepLength,w,step, Obstacles, oRot, oTrans)
clear comp
clear samples
clear coverage
clear totalCoverage
clear sumQueriesConnection
clear TestResultConnection
clear TotalConnectionTime 
clear sampleTime
stopCon = 50;
stopInterval = 50;
maxSamples = stopInterval +300;

[obsN notInUse2] = size(Obstacles);

%%%%%%%% Starting the Probabilistic RoadMap method%%%%%%%              
%Config(1:8,1:4,4,1000) = zeros;
comp(1) = struct('Node',[1], 'NB',[0]);
emptyComp(1) = 2;

con = 1;
tic %Start sampling timer
for pos =1:100000
    
    % Make a new random robot configuration
    [ManipD, ManD, fTD, lTD, lRotD, lTransD, DHD, linValuesD, thetaD] =...
        makeRandomRobotConfig();
    [linksN notInUse1] = size(ManipD);

    % Checking for collision  between links and obstacles
    collisionDummy = collisionCheckABB(linksN, obsN, ManipD, lRotD, lTransD, Obstacles, oRot, oTrans);
    
    % If the robot manipulator collide, make a new neighbour sample
    if collisionDummy
        conf(2:4,1) = linValuesD;
        conf(5:10,1)= thetaD;
        newConfig = moveRandStep(step,w,conf, LinLimits, JALimits);
        
        linValues = newConfig(2:4);
        theta = newConfig(5:10);

        [Manip, Man, fT, lT, lRot, lTrans, DH, LinLimits, JALimits] = makeRobot(linValues, theta );
        collision = collisionCheckABB(linksN, obsN, Manip, lRot, lTrans, Obstacles, oRot, oTrans);
        % if no collision detected in current configuration, store
        % the configuration
        if collision == false
%             drawRobot(1,[1 0 0], linksN, Manip, lRot, lTrans)
%             drawGantryCrane(1, fT, linValues(1))
            %                     theta = [Th1 Th2 Th3 Th4]';
            Samples(1,con) = 0;
            Samples(2:4,con) = linValues;
            Samples(5:10,con)= theta;
            Samples(11,con) = 0;

            %         % plot the configuration
            %         plot3( [fT(1,4,1) fT(1,4,2) fT(1,4,3) fT(1,4,4)],[fT(2,4,1) fT(2,4,2) fT(2,4,3) fT(2,4,4) ],[fT(3,4,1) fT(3,4,2) fT(3,4,3) fT(3,4,4)] )

            if con == 1
                Samples(11,con)=1; %Sample 1 belongs to component 1
            else
                clear TempMat;
                for teller = 1:con-1 %Computing distance to all nodes in  one component
                    TempMat(teller,:) = [configDist([ 0; linValues; theta], Samples(1:10,teller), w ),teller ];
                end

                % Sort the matrix with increasing distance
                clear sortMat;
                sortMat = sortrows(TempMat);
                % Try make a connection between current node
                % and closest node
                [rows col] = size(sortMat);
                if rows>K
                    rows = K;
                end

                Config(1:10,1) = [0; linValues; theta ];
                for ind=1:rows
                    c = Samples(11,sortMat(ind,2));
                    %                                     Config(1:10,1) = [0; linValues; theta ];
                    Config(1:10,2) = Samples(1:10, sortMat(ind,2));
                    interaction = binCollisionCheckABB( Config,linStepLength, angularStepLength,linksN, obsN, Obstacles, oRot, oTrans);
                    if ~interaction % If no interaction, add the node to the component
                        [ connectSuccess Samples comp emptyComp] = connectNodeABB_K(con, c, ind, Samples, comp, sortMat, emptyComp);
                        if connectSuccess % Node added to component break for loop
                            break;
                        end
                    end
                end % If no connection found , try next sample.

                % if component connection failed, make a new component
                [Samples comp emptyComp] = connectToEmptyComponentABB(con,Samples, comp, emptyComp);
            end
            con = con + 1;
            
            if con==stopCon % Do some testing
                interval = con/stopInterval;
                %%%%%%%% TESTING, STOP SAMPLE TIMER%%%%%%
                sampleTime(interval) = toc;
                %% Test the probabilistic road map
                
                [testConfigCovered, TestQueryConnection, ConnectionTime, PathSearchTime, nTestNodes ] = PRM_SamplingAnalysis(comp, Samples, w);
                
                numberOfComponents = length(comp);
                
                % Storing the data                
                
                if interval==1
                TotalComponents(interval) = numberOfComponents;
                TotalCoverage = testConfigCovered;
                TestResultConnection = TestQueryConnection;
                TotalConnectionTime(:,:,interval) = ConnectionTime; 
                TotalPathSearchTime(:,:,interval) = PathSearchTime;
                else
                TotalComponents(interval) = numberOfComponents;    
                TotalCoverage = TotalCoverage + testConfigCovered;
                TestResultConnection = TestResultConnection + TestQueryConnection;
                TotalConnectionTime(:,:,interval) = ConnectionTime; 
                TotalPathSearchTime(:,:,interval) = PathSearchTime;
                end
                                
                coverage(interval) = sum(testConfigCovered);
                                             
                sumColums = sum(TestQueryConnection);
                sumQueriesConnection(interval) = sum(sumColums);
                
                stopCon = stopCon + stopInterval;
                if stopCon == maxSamples
                    disp('Reached 300 samples')
                    return
                end
                tic %Start new smaple timer
            end %Done testing and collecting data
            
        end
    end %end the collisionDummy
end %end the pos loop

disp('Done with Gaussian sampling')
